=begin
###########################################################################
Copyright 2011, TIG
###########################################################################
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided the above
copyright notice appear in all copies.
###########################################################################
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
###########################################################################
Name: FixtreeD.rb
###########################################################################
Usage:
Edit a tree[d] group that you have imported with OBJimporter, so that the 
active_entities are that group's.
In the Ruby Console you type:
  FixtreeD.new
All coplanar edges will be removed.
All other edges are hidden, and no longer not cast/receive shadows.
The leaves' front materials are copied to their backs.
Hidden Geometry is sw1tched 'off'.
Progress is reported in the Status Bar and Ruby Console
It can take some time to process a complex tree that has 1000s of parts!
###########################################################################
Donations: By Paypal.com to info @ revitrev.org
###########################################################################
Version: 1.0. 20110204 First Issue.
###########################################################################
=end
require 'sketchup.rb'
###
class FixtreeD
  def initialize()
    @model=Sketchup.active_model
    @ents=@model.active_entities
    @view=@model.active_view
    if @ents.parent == @model
      UI.beep
      puts "You must be in the context of a group-edit (i.e. the tree[d] imported from an obj)...\nRetry..."
      return ii
    end#if
    @model.start_operation("FixtreeD")
      self.erase_coplanar_edges()
      self.hide_edges()
      self.back_materials()
      puts "Done.\n"
    @model.commit_operation()
    return ''
  end#def
  def erase_coplanar_edges()
    edges=[]
    @ents.each{|e|edges << e if e.valid? and e.class==Sketchup::Edge}
    counter=0 
    edges.each{|e|
      next if not e.valid?
      if not e.faces[0] ### no edges
        e.erase!
        counter+=1
        Sketchup::set_status_text("Coplanar Edges Erased = #{counter}",SB_PROMPT)
      end
      next if not e.valid?
      if e.faces.length == 2
        if e.faces[0].normal.dot(e.faces[1].normal) > 0.99999999 && e.faces[0].material==e.faces[1].material && e.faces[0].back_material==e.faces[1].back_material
          e.erase!
          counter+=1
          Sketchup::set_status_text("Coplanar Edges Erased = #{counter}",SB_PROMPT)
        end#if
      end#if
    }
    @view.refresh
    puts "Coplanar Edges Erased = #{counter}"
  end#def
  def hide_edges()
    edges=[]
    @ents.each{|e|edges << e if e.valid? and e.class==Sketchup::Edge}
    elen=edges.length
    counter=0 
    edges.each{|e|
      next if not e.valid?
      e.hidden=true
      e.casts_shadows=false
      e.receives_shadows=false
      counter+=1
      Sketchup::set_status_text("Edges Hidden = #{counter} of #{elen}",SB_PROMPT)
    }
    @model.rendering_options["DrawHidden"]=false
    @view.refresh
    puts "Edges Hidden = #{counter}"
  end#def
  def back_materials()
    texture_writer=Sketchup.create_texture_writer
    faces=[]
    @ents.each{|e|faces << e if e.valid? and e.class==Sketchup::Face}
    flen=faces.length
    counter=0 
    faces.each{|face|
      samples = []
      samples << face.vertices[0].position			   ### 0,0 | Origin
      samples << samples[0].offset(face.normal.axes.x) ### 1,0 | Offset Origin in X
	  samples << samples[0].offset(face.normal.axes.y) ### 0,1 | Offset Origin in Y
      samples << samples[1].offset(face.normal.axes.y) ### 1,1 | Offset X in Y
      xyz = [];uv = []### Arrays containing 3D and UV points.
      uvh = face.get_UVHelper(true, true, texture_writer)
      samples.each{|position|
		xyz << position ### XYZ 3D coordinates
		uvq = uvh.get_front_UVQ(position) ### UV 2D coordinates
		uv << self.flattenUVQ(uvq)
	  }
	  pts = [] ### Position texture.
	  (0..3).each{|i|
		pts << xyz[i]
		pts << uv[i]
	  }
      mat=face.material
      #face.reverse!
	  face.position_material(mat, pts, false)
      #face.reverse!
      counter+=1
      Sketchup::set_status_text("Faces Fixed = #{counter} of #{flen}",SB_PROMPT)
    }
    @view.refresh
    puts "Faces Fixed = #{counter}"
  end#def
  def flattenUVQ(uvq)
	return Geom::Point3d.new(uvq.x / uvq.z, uvq.y / uvq.z, 1.0)
  end#def
end#class
###